@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -33,7 +33,9 @@
 #endif
 
 #include "cut-loader.h"
+#include "cut-elf-loader.h"
 #include "cut-mach-o-loader.h"
+#include "cut-pe-loader.h"
 #include "cut-test-iterator.h"
 #include "cut-experimental.h"
 
@@ -56,7 +58,9 @@ struct _CutLoaderPrivate
     GList *symbols;
     GModule *module;
     CutBinaryType binary_type;
+    CutELFLoader *elf_loader;
     CutMachOLoader *mach_o_loader;
+    CutPELoader *pe_loader;
     gboolean keep_opening;
     gboolean enable_convenience_attribute_definition;
     gchar *base_directory;
@@ -117,7 +121,9 @@ cut_loader_init (CutLoader *loader)
 
     priv->so_filename = NULL;
     priv->binary_type = CUT_BINARY_TYPE_UNKNOWN;
+    priv->elf_loader = NULL;
     priv->mach_o_loader = NULL;
+    priv->pe_loader = NULL;
     priv->keep_opening = FALSE;
     priv->enable_convenience_attribute_definition = FALSE;
     priv->base_directory = NULL;
@@ -143,11 +149,21 @@ dispose (GObject *object)
         priv->so_filename = NULL;
     }
 
+    if (priv->elf_loader) {
+        g_object_unref(priv->elf_loader);
+        priv->elf_loader = NULL;
+    }
+
     if (priv->mach_o_loader) {
         g_object_unref(priv->mach_o_loader);
         priv->mach_o_loader = NULL;
     }
 
+    if (priv->pe_loader) {
+        g_object_unref(priv->pe_loader);
+        priv->pe_loader = NULL;
+    }
+
     if (priv->module) {
         if (!priv->keep_opening)
             g_module_close(priv->module);
@@ -434,10 +450,14 @@ cut_loader_support_attribute (CutLoader *loader)
     if (!priv->enable_convenience_attribute_definition)
         return FALSE;
 
-    if (priv->mach_o_loader) {
+    if (priv->elf_loader) {
+        return cut_elf_loader_support_attribute(priv->elf_loader);
+    } else if (priv->mach_o_loader) {
         return cut_mach_o_loader_support_attribute(priv->mach_o_loader);
+    } else if (priv->pe_loader) {
+        return cut_pe_loader_support_attribute(priv->pe_loader);
     } else {
-#ifdef HAVE_BFD_H
+#ifdef HAVE_LIBBFD
         return cut_loader_support_attribute_bfd(loader);
 #else
         return cut_loader_support_attribute_scan(loader);
@@ -448,10 +468,14 @@ cut_loader_support_attribute (CutLoader *loader)
 static GList *
 collect_symbols (CutLoaderPrivate *priv)
 {
-    if (priv->mach_o_loader) {
+    if (priv->elf_loader) {
+        return cut_elf_loader_collect_symbols(priv->elf_loader);
+    } else if (priv->mach_o_loader) {
         return cut_mach_o_loader_collect_symbols(priv->mach_o_loader);
+    } else if (priv->pe_loader) {
+        return cut_pe_loader_collect_symbols(priv->pe_loader);
     } else {
-#ifdef HAVE_BFD_H
+#ifdef HAVE_LIBBFD
         return collect_symbols_bfd(priv);
 #else
         return collect_symbols_scan(priv);
@@ -574,9 +598,8 @@ register_test (CutLoader *loader, CutTestCase *test_case,
     cut_test_set_base_directory(test, priv->base_directory);
 
     if (attributes_setup_function) {
-        CutTestContext *test_context, *original_test_context;
+        CutTestContext *test_context;
 
-        original_test_context = cut_test_context_current_get();
         if (data_setup_function) {
             test_context = cut_test_context_new(NULL, NULL, test_case,
                                                 CUT_TEST_ITERATOR(test), NULL);
@@ -584,9 +607,9 @@ register_test (CutLoader *loader, CutTestCase *test_case,
             test_context = cut_test_context_new(NULL, NULL, test_case,
                                                 NULL, test);
         }
-        cut_test_context_current_set(test_context);
+        cut_test_context_current_push(test_context);
         attributes_setup_function();
-        cut_test_context_current_set(original_test_context);
+        cut_test_context_current_pop();
         g_object_unref(test_case);
     }
 
@@ -621,10 +644,16 @@ create_test_case (CutLoader *loader)
 
     priv = CUT_LOADER_GET_PRIVATE(loader);
 
-    g_module_symbol(priv->module, "setup", (gpointer)&setup);
-    g_module_symbol(priv->module, "teardown", (gpointer)&teardown);
-    g_module_symbol(priv->module, "startup", (gpointer)&startup);
-    g_module_symbol(priv->module, "shutdown", (gpointer)&shutdown);
+#define GET_HOOK_FUNCTION(name)                                         \
+    if (!g_module_symbol(priv->module, "cut_" #name, (gpointer)&name))  \
+        g_module_symbol(priv->module, #name, (gpointer)&name)
+
+    GET_HOOK_FUNCTION(setup);
+    GET_HOOK_FUNCTION(teardown);
+    GET_HOOK_FUNCTION(startup);
+    GET_HOOK_FUNCTION(shutdown);
+
+#undef GET_HOOK_FUNCTION
 
     filename = g_path_get_basename(priv->so_filename);
     if (g_str_has_prefix(filename, "lib")) {
@@ -669,12 +698,24 @@ cut_loader_load_test_case (CutLoader *loader)
         return NULL;
     }
 
+    priv->elf_loader = cut_elf_loader_new(priv->so_filename);
+    if (!cut_elf_loader_is_elf(priv->elf_loader)) {
+        g_object_unref(priv->elf_loader);
+        priv->elf_loader = NULL;
+    }
+
     priv->mach_o_loader = cut_mach_o_loader_new(priv->so_filename);
     if (!cut_mach_o_loader_is_mach_o(priv->mach_o_loader)) {
         g_object_unref(priv->mach_o_loader);
         priv->mach_o_loader = NULL;
     }
 
+    priv->pe_loader = cut_pe_loader_new(priv->so_filename);
+    if (!cut_pe_loader_is_dll(priv->pe_loader)) {
+        g_object_unref(priv->pe_loader);
+        priv->pe_loader = NULL;
+    }
+
     priv->symbols = collect_symbols(priv);
     if (!priv->symbols)
         return NULL;
