@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -26,15 +26,17 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-#endif
-#ifdef HAVE_SYS_WAIT_H
-#  include <sys/wait.h>
-#endif
 #ifdef HAVE_IO_H
 #  include <io.h>
 #  define pipe(phandles) _pipe(phandles, 4096, _O_BINARY)
+#else
+#  ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#  endif
+#endif
+
+#ifdef HAVE_SYS_WAIT_H
+#  include <sys/wait.h>
 #endif
 
 #include <glib.h>
@@ -167,6 +169,8 @@ cut_pipeline_new_from_run_context (CutRunContext *run_context)
                         cut_run_context_get_multi_thread(run_context),
                         "max-threads",
                         cut_run_context_get_max_threads(run_context),
+                        "handle-signals",
+                        cut_run_context_get_handle_signals(run_context),
                         "exclude-files",
                         cut_run_context_get_exclude_files(run_context),
                         "exclude-directories",
@@ -196,7 +200,7 @@ cut_pipeline_error_quark (void)
     return g_quark_from_static_string("cut-pipeline-error-quark");
 }
 
-#define emit_error(pipeline, code, error, format, ...) do               \
+#define emit_error(pipeline, code, error, ...) do                       \
 {                                                                       \
     CutPipeline *_pipeline;                                             \
     CutPipelinePrivate *_priv;                                          \
@@ -207,7 +211,7 @@ cut_pipeline_error_quark (void)
     _run_context = CUT_RUN_CONTEXT(_pipeline);                          \
     cut_run_context_emit_error(_run_context, CUT_PIPELINE_ERROR,        \
                                code, error,                             \
-                               format, ## __VA_ARGS__);                 \
+                               __VA_ARGS__);                            \
     cut_run_context_emit_complete_run(_run_context, FALSE);             \
 } while (0)
 
