@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -53,12 +53,11 @@
 #include "cut-backtrace-entry.h"
 #include "cut-utils.h"
 
-#define cut_omit(context, message, ...) do                      \
+#define cut_omit(context, message) do                           \
 {                                                               \
     cut_test_context_register_result(context,                   \
                                      CUT_TEST_RESULT_OMISSION,  \
-                                     message, ## __VA_ARGS__,   \
-                                     NULL);                     \
+                                     message);                  \
     cut_test_context_long_jump(context);                        \
 } while (0)
 
@@ -87,6 +86,8 @@ struct _CutTestContextPrivate
     gchar *fixture_data_dir;
     GHashTable *cached_fixture_data;
     GList *backtrace;
+    gchar *user_message;
+    guint user_message_keep_count;
     GMutex *mutex;
 };
 
@@ -248,6 +249,9 @@ cut_test_context_init (CutTestContext *context)
 
     priv->backtrace = NULL;
 
+    priv->user_message = NULL;
+    priv->user_message_keep_count = 0;
+
     priv->mutex = g_mutex_new();
 }
 
@@ -262,6 +266,20 @@ free_data_list (CutTestContextPrivate *priv)
     priv->current_data = NULL;
 }
 
+static void
+clear_user_message (CutTestContextPrivate *priv)
+{
+    if (priv->user_message_keep_count > 0)
+        priv->user_message_keep_count--;
+
+    if (priv->user_message_keep_count == 0) {
+        if (priv->user_message) {
+            g_free(priv->user_message);
+            priv->user_message = NULL;
+        }
+    }
+}
+
 static void
 dispose (GObject *object)
 {
@@ -322,6 +340,8 @@ dispose (GObject *object)
         priv->backtrace = NULL;
     }
 
+    clear_user_message(priv);
+
     if (priv->mutex) {
         g_mutex_free(priv->mutex);
         priv->mutex = NULL;
@@ -408,16 +428,51 @@ cut_test_context_current_quit (void)
     g_static_private_free(&current_context_private);
 }
 
-void
-cut_test_context_current_set (CutTestContext *context)
+static void
+contexts_free (gpointer data)
 {
-    g_static_private_set(&current_context_private, context, NULL);
+    GPtrArray *contexts = data;
+    g_ptr_array_free(contexts, TRUE);
+}
+
+void
+cut_test_context_current_push (CutTestContext *context)
+{
+    GPtrArray *contexts;
+
+    contexts = g_static_private_get(&current_context_private);
+    if (!contexts) {
+        contexts = g_ptr_array_new();
+        g_static_private_set(&current_context_private, contexts, contexts_free);
+    }
+    g_ptr_array_add(contexts, context);
 }
 
 CutTestContext *
-cut_test_context_current_get (void)
+cut_test_context_current_pop (void)
 {
-    return g_static_private_get(&current_context_private);
+    GPtrArray *contexts;
+    CutTestContext *context = NULL;
+
+    contexts = g_static_private_get(&current_context_private);
+    if (contexts) {
+        context = g_ptr_array_index(contexts, contexts->len - 1);
+        g_ptr_array_remove_index(contexts, contexts->len - 1);
+    }
+
+    return context;
+}
+
+CutTestContext *
+cut_test_context_current_peek (void)
+{
+    GPtrArray *contexts;
+
+    contexts = g_static_private_get(&current_context_private);
+    if (contexts)
+        return g_ptr_array_index(contexts, contexts->len - 1);
+    else
+        return NULL;
 }
 
 CutTestContext *
@@ -455,6 +510,7 @@ cut_test_context_set_run_context (CutTestContext *context, CutRunContext *run_co
         g_object_unref(priv->run_context);
     if (run_context)
         g_object_ref(run_context);
+
     priv->run_context = run_context;
 }
 
@@ -692,6 +748,7 @@ cut_test_context_pass_assertion (CutTestContext *context)
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     g_return_if_fail(priv->test);
 
+    clear_user_message(priv);
     g_signal_emit_by_name(priv->test, "pass-assertion", context);
 }
 
@@ -731,35 +788,27 @@ cut_test_context_emit_signal (CutTestContext *context,
 }
 
 void
-cut_test_context_register_result_va_list (CutTestContext *context,
-                                          CutTestResultStatus status,
-                                          const char *system_message,
-                                          const char *user_message_format,
-                                          va_list args)
+cut_test_context_register_result (CutTestContext *context,
+                                  CutTestResultStatus status,
+                                  const char *system_message)
 {
     CutTestContextPrivate *priv;
     CutTestResult *result;
     CutTestData *test_data = NULL;
-    gchar *user_message = NULL;
 
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     if (cut_test_result_status_is_critical(status))
         priv->failed = TRUE;
 
-    if (user_message_format) {
-        user_message = g_strdup_vprintf(user_message_format, args);
-    }
-
     if (priv->current_data)
         test_data = priv->current_data->data;
     result = cut_test_result_new(status,
                                  priv->test, priv->test_iterator,
                                  priv->test_case, priv->test_suite,
                                  test_data,
-                                 user_message, system_message,
+                                 priv->user_message, system_message,
                                  priv->backtrace);
-    if (user_message)
-        g_free(user_message);
+    clear_user_message(priv);
 
     if (priv->test) {
         CutProcess *process;
@@ -804,22 +853,6 @@ cut_test_context_register_result_va_list (CutTestContext *context,
     g_object_unref(result);
 }
 
-void
-cut_test_context_register_result (CutTestContext *context,
-                                  CutTestResultStatus status,
-                                  const gchar *message,
-                                  ...)
-{
-    va_list args;
-    const gchar *user_message_format;
-
-    va_start(args, message);
-    user_message_format = va_arg(args, gchar *);
-    cut_test_context_register_result_va_list(context, status, message,
-                                             user_message_format, args);
-    va_end(args);
-}
-
 void
 cut_test_context_set_failed (CutTestContext *context, gboolean failed)
 {
@@ -921,7 +954,7 @@ const GError *
 cut_test_context_take_g_error (CutTestContext *context, GError *error)
 {
     return cut_test_context_take(context, error,
-                                 (CutDestroyFunction) g_error_free);
+                                 (CutDestroyFunction)g_error_free);
 }
 
 const GList *
@@ -962,8 +995,7 @@ cut_test_context_trap_fork (CutTestContext *context)
         cut_test_context_register_result(context,
                                          CUT_TEST_RESULT_OMISSION,
                                          "can't use cut_fork() "
-                                         "in multi thread mode",
-                                         NULL);
+                                         "in multi thread mode");
         cut_test_context_long_jump(context);
     }
 
@@ -1114,20 +1146,21 @@ cut_test_context_set_fixture_data_dir (CutTestContext *context,
         g_free(priv->fixture_data_dir);
 
     va_start(args, path);
-    priv->fixture_data_dir = cut_utils_expand_pathv(path, &args);
+    priv->fixture_data_dir = cut_utils_expand_path_va_list(path, args);
     va_end(args);
 }
 
 gchar *
-cut_test_context_build_fixture_data_pathv (CutTestContext *context,
-                                           const gchar *path, va_list *args)
+cut_test_context_build_fixture_data_path_va_list (CutTestContext *context,
+                                                  const gchar *path,
+                                                  va_list args)
 {
     CutTestContextPrivate *priv;
     gchar *concatenated_path, *full_path;
 
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
 
-    concatenated_path = cut_utils_build_pathv(path, args);
+    concatenated_path = cut_utils_build_path_va_list(path, args);
     if (g_path_is_absolute(concatenated_path)) {
         full_path = concatenated_path;
     } else {
@@ -1152,49 +1185,56 @@ cut_test_context_build_fixture_data_path (CutTestContext *context,
     gchar *full_path;
 
     va_start(args, path);
-    full_path = cut_test_context_build_fixture_data_pathv(context, path, &args);
+    full_path = cut_test_context_build_fixture_data_path_va_list(context,
+                                                                 path, args);
     va_end(args);
 
     return full_path;
 }
 
 const gchar *
-cut_test_context_get_fixture_data_stringv (CutTestContext *context,
-                                           GError **error,
-                                           const gchar *path,
-                                           va_list *args)
+cut_test_context_get_fixture_data_string_va_list (CutTestContext *context,
+                                                  GError **error,
+                                                  gchar **full_path,
+                                                  const gchar *path,
+                                                  va_list args)
 {
     CutTestContextPrivate *priv;
-    gchar *full_path;
     gpointer value;
+    gchar *_full_path;
 
     if (!path)
         return NULL;
 
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
 
-    full_path = cut_test_context_build_fixture_data_pathv(context, path, args);
-    value = g_hash_table_lookup(priv->cached_fixture_data, full_path);
-    if (value) {
-        g_free(full_path);
-    } else {
+    _full_path = cut_test_context_build_fixture_data_path_va_list(context,
+                                                                  path, args);
+    value = g_hash_table_lookup(priv->cached_fixture_data, _full_path);
+    if (!value) {
         gchar *contents;
         gsize length;
 
-        if (g_file_get_contents(full_path, &contents, &length, error)) {
-            g_hash_table_insert(priv->cached_fixture_data, full_path, contents);
+        if (g_file_get_contents(_full_path, &contents, &length, error)) {
+            g_hash_table_insert(priv->cached_fixture_data,
+                                g_strdup(_full_path),
+                                contents);
             value = contents;
-        } else {
-            g_free(full_path);
         }
     }
 
+    if (full_path)
+        *full_path = _full_path;
+    else
+        g_free(_full_path);
+
     return value;
 }
 
 const gchar *
 cut_test_context_get_fixture_data_string (CutTestContext *context,
                                           GError **error,
+                                          gchar **full_path,
                                           const gchar *path, ...)
 {
     const gchar *value;
@@ -1204,8 +1244,9 @@ cut_test_context_get_fixture_data_string (CutTestContext *context,
         return NULL;
 
     va_start(args, path);
-    value = cut_test_context_get_fixture_data_stringv(context, error,
-                                                      path, &args);
+    value = cut_test_context_get_fixture_data_string_va_list(context, error,
+                                                             full_path,
+                                                             path, args);
     va_end(args);
 
     return value;
@@ -1249,8 +1290,24 @@ cut_test_context_build_source_filename (CutTestContext *context,
         return g_strdup(filename);
 }
 
+static gchar *
+convert_directory_separator_in_relative_path (const gchar *relative_path)
+{
+    gchar *convert, *dirname, *basename;
+
+    dirname = g_path_get_dirname(relative_path);
+    basename = g_path_get_basename(relative_path);
+
+    convert = g_build_filename(dirname, basename, NULL);
+    g_free(dirname);
+    g_free(basename);
+
+    return convert;
+}
+
 void
 cut_test_context_push_backtrace (CutTestContext *context,
+                                 const char     *relative_path,
                                  const char     *filename,
                                  unsigned int    line,
                                  const char     *function_name,
@@ -1266,7 +1323,29 @@ cut_test_context_push_backtrace (CutTestContext *context,
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
 
     g_mutex_lock(priv->mutex);
-    full_filename = cut_test_context_build_source_filename(context, filename);
+    if (relative_path) {
+        const gchar *source_directory = NULL;
+        gchar *convert_relative_path;
+
+        convert_relative_path =
+            convert_directory_separator_in_relative_path(relative_path);
+
+        if (priv->run_context)
+            source_directory =
+                cut_run_context_get_source_directory(priv->run_context);
+        if (source_directory)
+            full_filename = g_build_filename(source_directory,
+                                             convert_relative_path,
+                                             filename,
+                                             NULL);
+        else
+            full_filename = g_build_filename(convert_relative_path,
+                                             filename, NULL);
+        g_free(convert_relative_path);
+    } else {
+        full_filename = cut_test_context_build_source_filename(context,
+                                                               filename);
+    }
     entry = cut_backtrace_entry_new(full_filename, line, function_name, info);
     g_free(full_filename);
     priv->backtrace = g_list_prepend(priv->backtrace, entry);
@@ -1316,6 +1395,74 @@ cut_test_context_get_last_backtrace (CutTestContext *context,
         *info = cut_backtrace_entry_get_info(entry);
 }
 
+void
+cut_test_context_set_user_message (CutTestContext *context,
+                                   const char     *format,
+                                   ...)
+{
+    va_list args;
+
+    va_start(args, format);
+    cut_test_context_set_user_message_va_list(context, format, args);
+    va_end(args);
+}
+
+void
+cut_test_context_set_user_message_backward_compatibility (CutTestContext *context,
+                                                          const char     *format,
+                                                          ...)
+{
+    va_list args;
+
+    if (!format)
+        return;
+
+    va_start(args, format);
+    cut_test_context_set_user_message_va_list(context, format, args);
+    va_end(args);
+}
+
+void
+cut_test_context_set_user_message_va_list (CutTestContext *context,
+                                           const char     *format,
+                                           va_list         args)
+{
+    CutTestContextPrivate *priv;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    clear_user_message(priv);
+    if (!format)
+        return;
+
+    priv->user_message = g_strdup_vprintf(format, args);
+}
+
+void
+cut_test_context_check_optional_assertion_message (CutTestContext *context,
+                                                   const char *message)
+{
+    CutTestContextPrivate *priv;
+
+    if (!message)
+        return;
+    if (message[0] == '\0')
+        return;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    clear_user_message(priv);
+    priv->user_message = g_strdup_printf("optional assertion message "
+                                         "on the current environment "
+                                         "is unavailable: <%s>", message);
+}
+
+void
+cut_test_context_keep_user_message (CutTestContext *context)
+{
+    CUT_TEST_CONTEXT_GET_PRIVATE(context)->user_message_keep_count++;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
