@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -29,11 +29,12 @@
 
 #include "cut-run-context.h"
 
-#include "cut-marshalers.h"
 #include "cut-test-result.h"
 #include "cut-utils.h"
+#include "cut-crash-backtrace.h"
 
 #include "../gcutter/gcut-error.h"
+#include "../gcutter/gcut-marshalers.h"
 
 #define CUT_TEST_ITERATOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_TEST_ITERATOR, CutTestIteratorPrivate))
 
@@ -118,12 +119,13 @@ cut_test_iterator_class_init (CutTestIteratorClass *klass)
 
     signals[START_TEST]
         = g_signal_new("start-iterated-test",
-                G_TYPE_FROM_CLASS(klass),
-                G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET(CutTestIteratorClass, start_iterated_test),
-                NULL, NULL,
-                _cut_marshal_VOID__OBJECT_OBJECT,
-                G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
+                       G_TYPE_FROM_CLASS(klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET(CutTestIteratorClass,
+                                       start_iterated_test),
+                       NULL, NULL,
+                       _gcut_marshal_VOID__OBJECT_OBJECT,
+                       G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
 
     signals[COMPLETE_TEST]
         = g_signal_new("complete-iterated-test",
@@ -132,7 +134,7 @@ cut_test_iterator_class_init (CutTestIteratorClass *klass)
                        G_STRUCT_OFFSET(CutTestIteratorClass,
                                        complete_iterated_test),
                        NULL, NULL,
-                       _cut_marshal_VOID__OBJECT_OBJECT_BOOLEAN,
+                       _gcut_marshal_VOID__OBJECT_OBJECT_BOOLEAN,
                        G_TYPE_NONE, 3,
                        CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT, G_TYPE_BOOLEAN);
 
@@ -231,7 +233,6 @@ typedef struct _RunTestInfo
     CutIteratedTest *iterated_test;
     CutTestContext *test_context;
     CutTestContext *parent_test_context;
-    CutTestContext *original_test_context;
 } RunTestInfo;
 
 static void
@@ -242,7 +243,10 @@ run_test_without_thread (gpointer data, gpointer user_data)
     CutTestCase *test_case;
     CutTestIterator *test_iterator;
     CutIteratedTest *iterated_test;
-    CutTestContext *test_context, *parent_test_context, *original_test_context;
+    CutTestContext *test_context, *parent_test_context;
+    gint signum;
+    jmp_buf crash_jump_buffer;
+    CutCrashBacktrace *crash_backtrace = NULL;
     gboolean *success = user_data;
 
     run_context = info->run_context;
@@ -251,23 +255,56 @@ run_test_without_thread (gpointer data, gpointer user_data)
     iterated_test = info->iterated_test;
     test_context = info->test_context;
     parent_test_context = info->parent_test_context;
-    original_test_context = info->original_test_context;
 
     if (cut_run_context_is_canceled(run_context))
         return;
 
-    cut_test_context_current_set(test_context);
+    cut_test_context_current_push(test_context);
 
-    g_signal_emit_by_name(test_iterator, "start-iterated-test",
-                          iterated_test, test_context);
-
-    cut_test_case_run_setup(test_case, test_context);
-    if (cut_test_context_is_failed(test_context)) {
-        *success = FALSE;
+    if (cut_run_context_is_multi_thread(run_context) ||
+        !cut_run_context_get_handle_signals(run_context)) {
+        signum = 0;
     } else {
-        if (!cut_test_run(CUT_TEST(iterated_test), test_context, run_context))
-            *success = FALSE;
+        crash_backtrace = cut_crash_backtrace_new(&crash_jump_buffer);
+        signum = setjmp(crash_jump_buffer);
     }
+    switch (signum) {
+    case 0:
+        g_signal_emit_by_name(test_iterator, "start-iterated-test",
+                              iterated_test, test_context);
+
+        cut_test_case_run_setup(test_case, test_context);
+        if (cut_test_context_is_failed(test_context)) {
+            *success = FALSE;
+        } else {
+            if (!cut_test_run(CUT_TEST(iterated_test),
+                              test_context, run_context))
+                *success = FALSE;
+        }
+
+        if (crash_backtrace)
+            cut_crash_backtrace_free(crash_backtrace);
+
+        break;
+#ifndef G_OS_WIN32
+    case SIGSEGV:
+    case SIGABRT:
+    case SIGTERM:
+        success = FALSE;
+        cut_crash_backtrace_emit(cut_run_context_get_test_suite(run_context),
+                                 test_case, CUT_TEST(iterated_test),
+                                 test_iterator,
+                                 cut_iterated_test_get_data(iterated_test),
+                                 test_context);
+        break;
+    case SIGINT:
+        cut_run_context_cancel(run_context);
+        break;
+#endif
+    default:
+        break;
+    }
+
     cut_test_case_run_teardown(test_case, test_context);
 
     cut_test_context_set_failed(parent_test_context,
@@ -278,7 +315,7 @@ run_test_without_thread (gpointer data, gpointer user_data)
 
     cut_test_context_set_test(test_context, NULL);
     cut_test_context_set_test(parent_test_context, NULL);
-    cut_test_context_current_set(original_test_context);
+    cut_test_context_current_pop();
 
     g_object_unref(run_context);
     g_object_unref(test_case);
@@ -286,7 +323,6 @@ run_test_without_thread (gpointer data, gpointer user_data)
     g_object_unref(iterated_test);
     g_object_unref(test_context);
     g_object_unref(parent_test_context);
-    g_object_unref(original_test_context);
     g_free(info);
 }
 
@@ -301,7 +337,7 @@ run_test_with_thread_support (CutTestIterator *test_iterator,
     RunTestInfo *info;
     CutTest *test;
     CutTestCase *test_case;
-    CutTestContext *original_test_context, *local_test_context;
+    CutTestContext *local_test_context;
     gboolean is_multi_thread;
     gboolean need_no_thread_run = TRUE;
     const gchar *multi_thread_attribute;
@@ -324,8 +360,6 @@ run_test_with_thread_support (CutTestIterator *test_iterator,
     cut_test_context_set_data(local_test_context,
                               cut_iterated_test_get_data(iterated_test));
 
-    original_test_context = cut_test_context_current_get();
-
     cut_test_context_set_test(test_context, test);
     cut_test_context_set_test(local_test_context, test);
 
@@ -336,7 +370,6 @@ run_test_with_thread_support (CutTestIterator *test_iterator,
     info->iterated_test = g_object_ref(iterated_test);
     info->test_context = local_test_context;
     info->parent_test_context = g_object_ref(test_context);
-    info->original_test_context = g_object_ref(original_test_context);
     if (is_multi_thread && thread_pool) {
         GError *error = NULL;
 
