@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,19 +18,23 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
+
 #include <glib.h>
 
 #include "cut-test.h"
 #include "cut-test-container.h"
 #include "cut-run-context.h"
-#include "cut-marshalers.h"
 #include "cut-test-result.h"
 #include "cut-utils.h"
+#include "cut-crash-backtrace.h"
+
+#include <gcutter/gcut-marshalers.h>
 
 #define CUT_TEST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_TEST, CutTestPrivate))
 
@@ -67,8 +71,8 @@ enum
     PENDING,
     NOTIFICATION,
     OMISSION,
+    CRASH,
     COMPLETE,
-    CRASHED,
     LAST_SIGNAL
 };
 
@@ -99,6 +103,10 @@ static gboolean     is_available (CutTest        *test,
 static void         invoke       (CutTest        *test,
                                   CutTestContext *test_context,
                                   CutRunContext  *run_context);
+static void         emit_result_signal
+                                 (CutTest        *test,
+                                  CutTestContext *test_context,
+                                  CutTestResult  *result);
 
 static void
 cut_test_class_init (CutTestClass *klass)
@@ -118,6 +126,7 @@ cut_test_class_init (CutTestClass *klass)
     klass->run = run;
     klass->is_available = is_available;
     klass->invoke = invoke;
+    klass->emit_result_signal = emit_result_signal;
 
     spec = g_param_spec_string("name",
                                "Test name",
@@ -170,7 +179,7 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutTestClass, success),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -180,7 +189,7 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutTestClass, failure),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -190,7 +199,7 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutTestClass, error),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -200,7 +209,7 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutTestClass, pending),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -210,7 +219,7 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutTestClass, notification),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -220,7 +229,17 @@ cut_test_class_init (CutTestClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET(CutTestClass, omission),
                         NULL, NULL,
-                        _cut_marshal_VOID__OBJECT_OBJECT,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
+                        G_TYPE_NONE, 2,
+                        CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
+
+    cut_test_signals[CRASH]
+        = g_signal_new("crash",
+                        G_TYPE_FROM_CLASS(klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET(CutTestClass, crash),
+                        NULL, NULL,
+                        _gcut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2,
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
@@ -230,17 +249,9 @@ cut_test_class_init (CutTestClass *klass)
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (CutTestClass, complete),
                        NULL, NULL,
-                       _cut_marshal_VOID__OBJECT_BOOLEAN,
+                       _gcut_marshal_VOID__OBJECT_BOOLEAN,
                        G_TYPE_NONE, 2, CUT_TYPE_TEST_CONTEXT, G_TYPE_BOOLEAN);
 
-    cut_test_signals[CRASHED]
-        = g_signal_new("crashed",
-                       G_TYPE_FROM_CLASS (klass),
-                       G_SIGNAL_RUN_LAST,
-                       G_STRUCT_OFFSET (CutTestClass, crashed),
-                       NULL, NULL,
-                       g_cclosure_marshal_VOID__STRING,
-                       G_TYPE_NONE, 1, G_TYPE_STRING);
 
     g_type_class_add_private(gobject_class, sizeof(CutTestPrivate));
 }
@@ -409,6 +420,14 @@ run (CutTest *test, CutTestContext *test_context, CutRunContext *run_context)
     CutTestPrivate *priv;
     gboolean success = TRUE;
     jmp_buf jump_buffer;
+    CutTestSuite *test_suite;
+    CutTestCase *test_case;
+    CutTestIterator *test_iterator;
+    CutTestResult *result;
+    CutTestData *data = NULL;
+    gint signum;
+    jmp_buf crash_jump_buffer;
+    CutCrashBacktrace *crash_backtrace = NULL;
 
     priv = CUT_TEST_GET_PRIVATE(test);
     klass = CUT_TEST_GET_CLASS(test);
@@ -416,30 +435,57 @@ run (CutTest *test, CutTestContext *test_context, CutRunContext *run_context)
     if (!klass->is_available(test, test_context, run_context))
         return FALSE;
 
-    g_signal_emit_by_name(test, "start", test_context);
+    test_suite = cut_test_context_get_test_suite(test_context);
+    test_case = cut_test_context_get_test_case(test_context);
+    test_iterator = cut_test_context_get_test_iterator(test_context);
+    if (CUT_IS_ITERATED_TEST(test))
+        data = cut_iterated_test_get_data(CUT_ITERATED_TEST(test));
 
-    cut_test_context_set_jump(test_context, &jump_buffer);
-    if (setjmp(jump_buffer) == 0) {
-        if (priv->timer) {
-            g_timer_start(priv->timer);
-        } else {
-            priv->timer = g_timer_new();
+    if (cut_run_context_is_multi_thread(run_context) ||
+        !cut_run_context_get_handle_signals(run_context)) {
+        signum = 0;
+    } else {
+        crash_backtrace = cut_crash_backtrace_new(&crash_jump_buffer);
+        signum = setjmp(crash_jump_buffer);
+    }
+    switch (signum) {
+    case 0:
+        g_signal_emit_by_name(test, "start", test_context);
+
+        cut_test_context_set_jump(test_context, &jump_buffer);
+        if (setjmp(jump_buffer) == 0) {
+            if (priv->timer) {
+                g_timer_start(priv->timer);
+            } else {
+                priv->timer = g_timer_new();
+            }
+            klass->invoke(test, test_context, run_context);
         }
-        klass->invoke(test, test_context, run_context);
+        g_timer_stop(priv->timer);
+
+        success = !cut_test_context_is_failed(test_context);
+
+        if (crash_backtrace)
+            cut_crash_backtrace_free(crash_backtrace);
+        break;
+#ifndef G_OS_WIN32
+    case SIGSEGV:
+    case SIGABRT:
+    case SIGTERM:
+        success = FALSE;
+        cut_crash_backtrace_emit(test_suite, test_case,
+                                 test, test_iterator, data,
+                                 test_context);
+        break;
+    case SIGINT:
+        cut_run_context_cancel(run_context);
+        break;
+#endif
+    default:
+        break;
     }
-    g_timer_stop(priv->timer);
 
-    success = !cut_test_context_is_failed(test_context);
     if (success) {
-        CutTestCase *test_case;
-        CutTestIterator *test_iterator;
-        CutTestResult *result;
-        CutTestData *data = NULL;
-
-        test_case = cut_test_context_get_test_case(test_context);
-        test_iterator = cut_test_context_get_test_iterator(test_context);
-        if (CUT_IS_ITERATED_TEST(test))
-            data = cut_iterated_test_get_data(CUT_ITERATED_TEST(test));
         result = cut_test_result_new(CUT_TEST_RESULT_SUCCESS,
                                      test, test_iterator, test_case,
                                      NULL, data,
@@ -686,23 +732,39 @@ cut_test_to_xml_string (CutTest *test, GString *string, guint indent)
 }
 
 void
-cut_test_emit_result_signal (CutTest *test,
-                             CutTestContext *test_context,
-                             CutTestResult *result)
+cut_test_set_result_elapsed (CutTest *test, CutTestResult *result)
 {
     CutTestPrivate *priv;
-    const gchar *status_signal_name = NULL;
-    CutTestResultStatus status;
 
     priv = CUT_TEST_GET_PRIVATE(test);
     cut_test_result_set_start_time(result, &(priv->start_time));
     cut_test_result_set_elapsed(result, cut_test_get_elapsed(test));
+}
+
+static void
+emit_result_signal (CutTest *test,
+                    CutTestContext *test_context,
+                    CutTestResult *result)
+{
+    const gchar *status_signal_name = NULL;
+    CutTestResultStatus status;
 
     status = cut_test_result_get_status(result);
     status_signal_name = cut_test_result_status_to_signal_name(status);
     g_signal_emit_by_name(test, status_signal_name, test_context, result);
 }
 
+void
+cut_test_emit_result_signal (CutTest *test,
+                             CutTestContext *test_context,
+                             CutTestResult *result)
+{
+    cut_test_set_result_elapsed(test, result);
+
+    return CUT_TEST_GET_CLASS(test)->emit_result_signal(test, test_context,
+                                                        result);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
