@@ -17,6 +17,7 @@ module RD
     VERSION = Version.new_from_version_string(SYSTEM_NAME, SYSTEM_VERSION)
 
     @@po = nil
+    @@name = nil
     class << self
       def version
         VERSION
@@ -25,6 +26,10 @@ module RD
       def po=(po)
         @@po = po
       end
+
+      def name=(name)
+        @@name = name
+      end
     end
 
     # must-have constants
@@ -36,6 +41,9 @@ module RD
     def initialize
       super
       @translate_data = nil
+      @n_sections = 0
+      @n_text_blocks_in_section = 0
+      @man_style = false
     end
 
     def apply_to_DocumentElement(element, contents)
@@ -51,8 +59,21 @@ module RD
     end
 
     def apply_to_Headline(element, title)
-      if element.level == 1
-        @title, @purpose = title.join.split(/\s*\B---\B\s*/, 2)
+      case element.level
+      when 1
+        title = title.join
+        @title, @purpose = title.split(/\s*\B---\B\s*/, 2)
+        if @purpose.nil?
+          components = title.split(/\s*\B\/\B\s*/)
+          if components.size == 3
+            name, package, manual = components
+            @title = name
+            @man_style = true
+          end
+        end
+      when 2
+        @n_sections += 1
+        @n_text_blocks_in_section = 0
       end
       [:headline, element.level - 1, title]
     end
@@ -66,7 +87,9 @@ module RD
     end
 
     def apply_to_Reference_with_RDLabel(element, contents)
-      raise "label with filename is unsupported" if element.label.filename
+      if element.label.filename
+        raise "label with filename is unsupported: #{element.label.inspect}"
+      end
       url = element.label.element_label
       label = contents.join("").chomp
       label = url if label.empty?
@@ -77,22 +100,32 @@ module RD
           url = "#{url}\##{anchor}"
         end
       when /\.html#[a-zA-Z\-_]+$/
-      when /\.png$/
-        return tag("inlinegraphic", {:fileref => url, :format => "PNG"})
+      when /\.svg$/, /\.png$/
+        url = url.sub(/svg$/, "png")
+        return tag("inlinemediaobject",
+                   {},
+                   [
+                    tag("imageobject", {},
+                        tag("imagedata",
+                            {
+                              :fileref => url.sub(/svg$/, "png"),
+                              :format => "PNG"
+                            })),
+                    tag("textobject", {}, tag("phrase", {}, label)),
+                   ])
       else
-        url = remove_lang_suffix(url.downcase) + ".html"
+        return tag("link",
+                   {:linkend => ref_entry_id(url)},
+                   remove_suffix(remove_lang_suffix(label)))
       end
-      label = remove_lang_suffix(label)
+      label = remove_suffix(remove_lang_suffix(label))
       tag("ulink", {:url => url}, label)
     end
 
     def apply_to_Reference_with_URL(element, contents)
       url = element.label.url
       label = contents.join("").chomp
-      if /\Ahttp:\/\/cutter\.(?:sf|sourceforge)\.net\/reference\// =~ url
-        url = File.basename($POSTMATCH)
-      end
-      url = label if label.empty?
+      label = url if label.empty?
       tag("ulink", {:url => url}, label)
     end
 
@@ -123,9 +156,26 @@ module RD
     end
 
     def apply_to_TextBlock(element, contents)
+      @n_text_blocks_in_section += 1
+      if @man_style and @n_sections == 1 and @n_text_blocks_in_section == 1
+        @purpose = contents.join.split(/\s+-\s+/, 2)[1]
+      end
       tag("para", {}, *contents)
     end
 
+    def apply_to_Emphasis(element, contents)
+      tag("emphasis", {}, *contents)
+    end
+
+    def apply_to_Footnote(element, contents)
+      tag("footnote", {},
+          tag("para", {}, *contents))
+    end
+
+    def apply_to_Code(element, contents)
+      tag("code", {}, *contents)
+    end
+
     def apply_to_StringElement(element)
       apply_to_String(element.content.chomp)
     end
@@ -162,8 +212,9 @@ module RD
         "  \"http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd\">"
     end
 
-    def ref_entry_id
-      File.basename(ARGF.filename, ".*").downcase
+    def ref_entry_id(filename=nil)
+      filename ||= ARGF.filename
+      remove_suffix(File.basename(filename)).downcase
     end
 
     def tag(name, attributes={}, *contents)
@@ -198,7 +249,7 @@ module RD
                 "id" => "#{ref_entry_id}.top_of_page",
               },
               @title),
-          tag("refmiscinfo", {}, h(_("CUTTER Library"))))
+          tag("refmiscinfo", {}, h(_("#{@@name || 'MY'} Library"))))
     end
 
     def ref_name_div
@@ -262,13 +313,21 @@ module RD
     end
 
     def remove_lang_suffix(string)
-      string.sub(/\.[a-z]{2}$/, "")
+      string.sub(/\.[a-z]{2}\z/, "")
+    end
+
+    def remove_suffix(string)
+      string.sub(/(?:\.[^.]+)+\z/, "")
     end
   end
 end
 
 $Visitor_Class = RD::RD2RefEntryVisitor
 ARGV.options do |opts|
+  opts.on("--name=NAME", "The library name") do |name|
+    RD::RD2RefEntryVisitor.name = name
+  end
+
   opts.on("-pPO", "--po=PO", "PO file") do |po|
     RD::RD2RefEntryVisitor.po = po
   end
