@@ -4,7 +4,12 @@ export BASE_DIR="`dirname $0`"
 top_dir="$BASE_DIR/.."
 
 if test x"$NO_MAKE" != x"yes"; then
-    make -C $top_dir/ > /dev/null || exit 1
+    if which gmake > /dev/null; then
+	MAKE=${MAKE:-"gmake"}
+    else
+	MAKE=${MAKE:-"make"}
+    fi
+    $MAKE -C $top_dir/ > /dev/null || exit 1
 fi
 
 if test -z "$CUTTER"; then
@@ -35,9 +40,25 @@ CUTTER_ARGS="$CUTTER_ARGS --exclude-directory lib"
 if echo "$@" | grep -- --mode=analyze > /dev/null; then
     :
 else
-    CUTTER_ARGS="$CUTTER_ARGS --stream=xml --stream-log-directory $top_dir/log"
+    CUTTER_ARGS="$CUTTER_ARGS --stream=xml --stream-directory $top_dir/log"
 fi
 if test x"$USE_GTK" = x"yes"; then
     CUTTER_ARGS="-u gtk $CUTTER_ARGS"
 fi
+
+case `uname` in
+    CYGWIN*)
+	PATH="$top_dir/cutter/.libs:$PATH"
+	PATH="$top_dir/gcutter/.libs:$PATH"
+	PATH="$top_dir/gdkcutter-pixbuf/.libs:$PATH"
+	PATH="$top_dir/test/lib/.libs:$PATH"
+	;;
+    Darwin)
+	export DYLD_LIBRARY_PATH="$top_dir/gdkcutter-pixbuf/.libs:$DYLD_LIBRARY_PATH"
+	;;
+    *)
+	:
+	;;
+esac
+
 $CUTTER_WRAPPER $CUTTER $CUTTER_ARGS "$@" $BASE_DIR
